import {ChangeDetectionStrategy, Component, EventEmitter, Input, Output, ViewChild} from "@angular/core";
import {MatPaginator, PageEvent} from "@angular/material";
import {PagingModel} from "@app/shared/models/paging.model";
import {environment} from "../../../../environments/environment";

@Component({
  selector: "dlcm-paginator",
  templateUrl: "./paginator.component.html",
  styleUrls: ["./paginator.component.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class PaginatorComponent {
  @Input()
  pageSizeOptions: number[];

  @Input()
  pagingModel: PagingModel;

  @Output()
  pageEvent: EventEmitter<PagingModel>;

  @ViewChild("paginator")
  paginator: MatPaginator;

  constructor() {
    this.pageEvent = new EventEmitter<PagingModel>();
    this.getDefaultValue();
  }

  private getDefaultValue(): void {
    this.pageSizeOptions = environment.pageSizeOptions;
  }

  onPageEvent(pageEventMaterial: PageEvent): void {
    this.pageEvent.emit(this.adaptMaterialPageEventToPagingModel(pageEventMaterial));
  }

  adaptMaterialPageEventToPagingModel(pageEventMaterial: PageEvent): PagingModel {
    return {
      length: pageEventMaterial.length,
      pageIndex: pageEventMaterial.pageIndex,
      pageSize: pageEventMaterial.pageSize,
    } as PagingModel;
  }
}
