import {ChangeDetectionStrategy, Component, EventEmitter, Input, OnInit, Output} from "@angular/core";
import {FormBuilder, FormGroup} from "@angular/forms";
import {AbstractComponent} from "@app/shared/components/abstract/abstract.component";
import {distinctUntilChanged} from "rxjs/internal/operators/distinctUntilChanged";
import {tap} from "rxjs/internal/operators/tap";
import {debounceTime} from "rxjs/operators";

@Component({
  selector: "dlcm-search",
  templateUrl: "./search.component.html",
  styleUrls: ["./search.component.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class SearchComponent extends AbstractComponent implements OnInit {
  searchForm: FormGroup;
  search: string = "search";

  @Input()
  value: string;

  @Output()
  valueEvent: EventEmitter<string>;

  private debounceTime: number = 600;

  constructor(private fb: FormBuilder) {
    super();
    this.valueEvent = new EventEmitter<string>();
  }

  ngOnInit(): void {
    this.searchForm = this.fb.group({
      [this.search]: this.value === null || this.value === undefined ? "" : this.value,
    });

    this.subscribe(
      this.searchForm.get(this.search).valueChanges
        .pipe(
          debounceTime(this.debounceTime),
          distinctUntilChanged(),
          tap((value: string) => {
            this.valueEvent.emit(value.trim().toLocaleLowerCase());
          }),
        ),
    );
  }

}
