import {ChangeDetectionStrategy, Component, EventEmitter, forwardRef, Input, OnInit, Output} from "@angular/core";
import {ControlValueAccessor, FormControl, NG_VALUE_ACCESSOR} from "@angular/forms";
import {MatSelectChange} from "@angular/material";
import {AbstractComponent} from "@app/shared/components/abstract/abstract.component";
import {StringUtil} from "@app/core/utils/string.util";
import _ from "lodash";

@Component({
  selector: "dlcm-searchable-multi-select",
  templateUrl: "./searchable-multi-select.component.html",
  styleUrls: ["./searchable-multi-select.component.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
  providers: [
    {
      provide: NG_VALUE_ACCESSOR,
      multi: true,
      useExisting: forwardRef(() => SearchableMultiSelectComponent),
    },
  ],
})
export class SearchableMultiSelectComponent extends AbstractComponent implements OnInit, ControlValueAccessor {
  @Input()
  placeholder: string;

  @Input()
  list: string[];

  @Input()
  valueKey: string;

  @Input()
  labelKey: string;

  @Input()
  formControl: FormControl;

  @Output()
  valueChangeEvent: EventEmitter<any[]>;

  constructor() {
    super();
    this.valueChangeEvent = new EventEmitter<any[]>();
  }

  ngOnInit(): void {
  }

  getCorrespondingLabel(valueElement: string | undefined): string {
    if (valueElement === undefined || this.list.length === 0) {
      return StringUtil.stringEmpty;
    }
    return this.list.find(v => v[this.valueKey] === valueElement)[this.labelKey];
  }

  getCorrespondingLabelForOthers(): string {
    const listOtherValue = _.slice(this.formControl.value, 1);
    const labelOther: string[] = [];
    listOtherValue.forEach(o => labelOther.push(this.getCorrespondingLabel(o)));
    return _.join(labelOther, ", ");
  }

  selectionChange($event: MatSelectChange): void {
    this.propagateChange($event.value);
    this.valueChangeEvent.emit($event.value);
  }

  propagateChange = (__: any) => {
  };

  registerOnChange(fn: any): void {
    this.propagateChange = fn;
  }

  registerOnTouched(fn: any): void {
  }

  setDisabledState(isDisabled: boolean): void {
  }

  writeValue(value: string[]): void {
    // if (value) {
    // this.formControl.setValue(value);
    // }
  }
}
