import {BaseStoreNameSpace} from "@app/core/models/stores/base-store-namespace.model";
import {StoreActionClass} from "@app/core/models/stores/state-action.model";
import {SolidifyMetadataUtil} from "@app/core/utils/stores/solidify-metadata.util";
import {ActionOptions} from "@ngxs/store";

export function RegisterDefaultAction<T extends BaseStoreNameSpace>(defaultActionClassCallback: DefaultActionClassCallback<T>, options?: ActionOptions): MethodDecorator {
  return (target: any, name: string, descriptor: TypedPropertyDescriptor<any>) => {
    const meta = SolidifyMetadataUtil.ensureStoreSolidifyMetadataBaseClass(target.constructor);
    if (!Array.isArray(meta.defaultActions)) {
      meta.defaultActions = [];
    }
    meta.defaultActions.push({
      fn: name,
      callback: defaultActionClassCallback,
      options,
    });
  };
}

export function OverrideDefaultAction(): MethodDecorator {
  return (target: any, name: string, descriptor: TypedPropertyDescriptor<any>) => {
    const meta = SolidifyMetadataUtil.ensureStoreSolidifyMetadata(target.constructor);
    if (!Array.isArray(meta.excludedRegisteredDefaultActionFns)) {
      meta.excludedRegisteredDefaultActionFns = [];
    }
    meta.excludedRegisteredDefaultActionFns.push(name);
  };
}

export interface RegisteredDefaultAction<T extends BaseStoreNameSpace> {
  fn?: string;
  callback?: DefaultActionClassCallback<T>;
  options?: ActionOptions;
}

export type DefaultActionClassCallback<T extends BaseStoreNameSpace> = (baseNameSpace?: T) => StoreActionClass;
