import {Inject, OnInit} from "@angular/core";
import {MAT_DIALOG_DATA, MatDialogRef} from "@angular/material";
import {AbstractComponent} from "@app/shared/components/abstract/abstract.component";
import {ResourceNameSpace} from "../../../core/stores/resource/resource-namespace.model";
import {ResourceStateModel} from "@app/core/stores/resource/resource.state";
import {StateEnum} from "@app/shared/enums/state.enum";
import {DeleteDialogModel} from "@app/shared/models/delete-dialog.model";
import {ResourceActionHelper} from "@app/core/stores/resource/resource-action.helper";
import {Store} from "@ngxs/store";

export abstract class AbstractDeleteDialog<T, U extends ResourceStateModel<T>> extends AbstractComponent implements OnInit {
  paramMessage: { name: string } = {name: ""};

  public abstract readonly KEY_TITLE: string;
  public abstract readonly KEY_MESSAGE: string;
  public abstract readonly KEY_CONFIRM_BUTTON: string;
  public abstract readonly KEY_CANCEL_BUTTON: string;

  protected constructor(protected store: Store,
                        protected dialogRef: MatDialogRef<AbstractDeleteDialog<T, U>>,
                        @Inject(MAT_DIALOG_DATA) public data: DeleteDialogModel,
                        protected state: StateEnum,
                        private resourceActionNameSpace: ResourceNameSpace) {
    super();
  }

  ngOnInit(): void {
    this.paramMessage.name = this.data.name;
  }

  confirm(): void {
    this.store.dispatch(ResourceActionHelper.delete(this.resourceActionNameSpace, this.data.resId));
    this.dialogRef.close();
  }
}
