import {ApiModuleEnum} from "@app/shared/enums/api-module.enum";
import {ApiResourceNameEnum} from "@app/shared/enums/api-resource-name.enum";
import {environment} from "../../../environments/environment";

export class ApiKeyword {
  static PARENT_ID: string = "parentId";
  static SUB_RESOURCE: string = "subResource";
}

const sep = "/";
const subResource = `{${ApiKeyword.SUB_RESOURCE}}`;
const parentId = `{${ApiKeyword.PARENT_ID}}`;
const parentIdInUrl = sep + parentId + sep;
const search = "search";
const api = "api";
const apiInUrl = sep + api + sep;

class BaseResourceApiEnum {
  static access: string = (environment.production === true ? environment.ACCESS_APPLICATION : apiInUrl) + ApiModuleEnum.ACCESS;
  static admin: string = (environment.production === true ? environment.ADMIN_APPLICATION : apiInUrl) + ApiModuleEnum.ADMIN;
  static dataMgmt: string = (environment.production === true ? environment.ADMIN_APPLICATION : apiInUrl) + ApiModuleEnum.DATAMGMT;
  static resourceSrv: string = (environment.production === true ? environment.ADMIN_APPLICATION : apiInUrl) + ApiModuleEnum.RES_SRV;
  static preservationPlanning: string = (environment.production === true ? environment.ADMIN_APPLICATION : apiInUrl) + ApiModuleEnum.PRES_PLANNING;
  static ingest: string = (environment.production === true ? environment.INGEST_APPLICATION : apiInUrl) + ApiModuleEnum.INGEST;
  static preingest: string = (environment.production === true ? environment.INGEST_APPLICATION : apiInUrl) + ApiModuleEnum.PREINGEST;
  static archivalStorage: string = (environment.production === true ? environment.STORAGION_APPLICATION : apiInUrl) + ApiModuleEnum.ARCHIVALSTORAGE;
}

export interface ResourceApiEnum {
}

export interface SubResourceApiEnum extends ResourceApiEnum {
}

export class AccessResourceApiEnum implements ResourceApiEnum {
  static root: string =
    BaseResourceApiEnum.access;
  static dip: string =
    BaseResourceApiEnum.access + sep + ApiResourceNameEnum.DIP;
  static allMetadata: string =
    BaseResourceApiEnum.access + sep + ApiResourceNameEnum.ALL_METADATA;
  static publicMetadata: string =
    BaseResourceApiEnum.access + sep + ApiResourceNameEnum.PUBLIC_METADATA;
  static privateMetadata: string =
    BaseResourceApiEnum.access + sep + ApiResourceNameEnum.PRIVATE_METADATA;
  static publicMetadataSearch: string =
    AccessResourceApiEnum.publicMetadata + sep + search;
  static allMetadataSearch: string =
    AccessResourceApiEnum.allMetadata + sep + search;
  static privateMetadataSearch: string =
    AccessResourceApiEnum.privateMetadata + sep + search;
  static oaiSets: string =
    BaseResourceApiEnum.access + sep + ApiResourceNameEnum.OAI_SETS;
  static orders: string =
    BaseResourceApiEnum.access + sep + ApiResourceNameEnum.ORDER;
  static organizationalUnits: string =
    BaseResourceApiEnum.access + sep + ApiResourceNameEnum.ORG_UNIT;
}

export class DataManagementApiEnum implements ResourceApiEnum {
  static root: string =
    BaseResourceApiEnum.dataMgmt;
  static publicDiSearch: string =
    BaseResourceApiEnum.dataMgmt + sep + ApiResourceNameEnum.DI_PUBLIC + sep + search;
}

export class AdminResourceApiEnum implements ResourceApiEnum {
  // Resource
  static root: string =
    BaseResourceApiEnum.admin;
  static fundingAgencies: string =
    BaseResourceApiEnum.admin + sep + ApiResourceNameEnum.FUNDING_AGENCY;
  static institutions: string =
    BaseResourceApiEnum.admin + sep + ApiResourceNameEnum.INSTITUTION;
  static languages: string =
    BaseResourceApiEnum.admin + sep + ApiResourceNameEnum.LANGUAGE;
  static licenses: string =
    BaseResourceApiEnum.admin + sep + ApiResourceNameEnum.LICENSE;
  static oauth2Clients: string =
    BaseResourceApiEnum.admin + sep + ApiResourceNameEnum.OAUTH2_CLIENT;
  static organizationalUnits: string =
    BaseResourceApiEnum.admin + sep + ApiResourceNameEnum.ORG_UNIT;
  static people: string =
    BaseResourceApiEnum.admin + sep + ApiResourceNameEnum.PERSON;
  static preservationPolicies: string =
    BaseResourceApiEnum.admin + sep + ApiResourceNameEnum.PRES_POLICY;
  static roles: string =
    BaseResourceApiEnum.admin + sep + ApiResourceNameEnum.ROLE;
  static submissionAgreements: string =
    BaseResourceApiEnum.admin + sep + ApiResourceNameEnum.SUBMISSION_AGREEMENT;
  static submissionPolicies: string =
    BaseResourceApiEnum.admin + sep + ApiResourceNameEnum.SUB_POLICY;
  static users: string =
    BaseResourceApiEnum.admin + sep + ApiResourceNameEnum.USER;
}

export class AdminSubResourceApiEnum implements SubResourceApiEnum {
  // Sub Resource
  static organizationalUnitsSubResource: string =
    AdminResourceApiEnum.organizationalUnits + parentIdInUrl + subResource;
  static organizationalUnitsSubmissionPolicy: string =
    AdminResourceApiEnum.organizationalUnits + parentIdInUrl + ApiResourceNameEnum.SUB_POLICY;
  static organizationalUnitsSubmissionAgreement: string =
    AdminResourceApiEnum.organizationalUnits + parentIdInUrl + ApiResourceNameEnum.SUBMISSION_AGREEMENT;
}

export class ArchivalStorageResourceApiEnum implements ResourceApiEnum {
  static root: string =
    BaseResourceApiEnum.archivalStorage;
  static aip: string =
    BaseResourceApiEnum.archivalStorage + sep + ApiResourceNameEnum.AIP;
  static storedAip: string =
    BaseResourceApiEnum.archivalStorage + sep + ApiResourceNameEnum.STORED_AIP;
}

export class IngestResourceApiEnum implements ResourceApiEnum {
  static root: string =
    BaseResourceApiEnum.ingest;
  static sip: string =
    BaseResourceApiEnum.ingest + sep + ApiResourceNameEnum.SIP;
}

export class PreIngestResourceApiEnum implements ResourceApiEnum {
  static root: string =
    BaseResourceApiEnum.preingest;
  static deposits: string =
    BaseResourceApiEnum.preingest + sep + ApiResourceNameEnum.DEPOSIT;
}

export class PreservationPlanningResourceApiEnum implements ResourceApiEnum {
  static root: string =
    BaseResourceApiEnum.preservationPlanning;
  static preservationJobs: string =
    BaseResourceApiEnum.preservationPlanning + sep + ApiResourceNameEnum.PRES_JOB;
}

export class ResourceSrvResourceApiEnum implements ResourceApiEnum {
  static root: string =
    BaseResourceApiEnum.resourceSrv;
  static applicationRoles: string =
    BaseResourceApiEnum.resourceSrv + sep + ApiResourceNameEnum.ROLE_APPLICATION;
  static basicAuthClients: string =
    BaseResourceApiEnum.resourceSrv + sep + ApiResourceNameEnum.BASIC_AUTH_CLIENT;
}
