import {CommonModule} from "@angular/common";
import {NgModule} from "@angular/core";
import {FormsModule, ReactiveFormsModule} from "@angular/forms";
import {MaterialModule} from "@app/material.module";
import {DataTableComponent} from "@app/shared/components/data-table/data-table.component";
import {SearchComponent} from "@app/shared/components/search/search.component";
import {SearchableMultiSelectComponent} from "@app/shared/components/searchable-multi-select/searchable-multi-select.component";
import {SearchableSingleSelectComponent} from "@app/shared/components/searchable-single-select/searchable-single-select.component";
import {LanguageState} from "@app/shared/stores/language/language.state";
import {LicenseState} from "@app/shared/stores/license/license.state";
import {OrganizationalUnitSubmissionPolicyState} from "@app/shared/stores/organizational-unit/submission-policy/organizational-unit-submission-policy.state";
import {OrganizationalUnitState} from "@app/shared/stores/organizational-unit/organizational-unit.state";
import {isPipes} from "@app/shared/pipes/is-pipes/is-pipes.model";
import {PreservationPolicyState} from "@app/shared/stores/preservation-policy/preservation-policy.state";
import {SharedState} from "@app/shared/stores/shared/shared.state";
import {SubmissionPolicyState} from "@app/shared/stores/submission-policy/submission-policy.state";
import {FontAwesomeModule} from "@fortawesome/angular-fontawesome";
import {library} from "@fortawesome/fontawesome-svg-core";
import {fas} from "@fortawesome/free-solid-svg-icons";
import {TranslateModule} from "@ngx-translate/core";
import {NgxsModule} from "@ngxs/store";
import {TableModule} from "primeng/table";
import {BreadcrumbComponent} from "./components/breadcrumb/breadcrumb.component";
import {PaginatorComponent} from "./components/paginator/paginator.component";
import { SearchScopeComponent } from "./components/search-scope/search-scope.component";
import {MatRadioModule} from "@angular/material";

const views = [];
const dialogs = [];
const components = [
  SearchComponent,
  DataTableComponent,
  PaginatorComponent,
  BreadcrumbComponent,
  SearchableSingleSelectComponent,
  SearchableMultiSelectComponent,
  SearchScopeComponent
];
const directives = [];
const modules = [
  CommonModule,
  FormsModule,
  ReactiveFormsModule,
  MaterialModule,
  FontAwesomeModule,
  TableModule,
  TranslateModule,
];

@NgModule({
  declarations: [
    ...views,
    ...dialogs,
    ...components,
    ...directives,
    ...isPipes,
  ],
  imports: [
    ...modules,
    NgxsModule.forFeature([
      SharedState,
      LicenseState,
      LanguageState,
      PreservationPolicyState,
      SubmissionPolicyState,
      OrganizationalUnitState,
      OrganizationalUnitSubmissionPolicyState,
    ]),
    MatRadioModule,
  ],
  exports: [
    ...modules,
    ...views,
    ...dialogs,
    ...components,
    ...directives,
    ...isPipes,
  ],
})
export class SharedModule {
}

// Fontawesome library
library.add(fas);
