import {LanguagesModel} from "@app/generated-api";
import {ResourceNameSpace} from "../../../core/stores/resource/resource-namespace.model";
import {ResourceState, ResourceStateModel, defaultResourceStateInitValue} from "@app/core/stores/resource/resource.state";
import {ApiResourceNameEnum} from "@app/shared/enums/api-resource-name.enum";
import {AdminResourceApiEnum, ResourceApiEnum} from "@app/shared/enums/api.enum";
import {StateEnum} from "@app/shared/enums/state.enum";
import {languageActionNameSpace} from "@app/shared/stores/language/language.action";
import {QueryParametersModel} from "@app/shared/models/query-parameters.model";
import {ApiService} from "@app/core/http/api.service";
import {NotificationService} from "@app/core/services/notification.service";
import {State, Store} from "@ngxs/store";

export interface LanguageStateModel extends ResourceStateModel<LanguagesModel> {
}

@State<LanguageStateModel>({
  name: StateEnum.language,
  defaults: {
    ...defaultResourceStateInitValue,
  },
})
export class LanguageState extends ResourceState<LanguagesModel> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService) {
    super(apiService, store, notificationService);
  }

  defineState(): StateEnum {
    return StateEnum.language;
  }

  defineResourceNameSpace(): ResourceNameSpace {
    return languageActionNameSpace;
  }

  defineUrlResource(): ResourceApiEnum {
    return AdminResourceApiEnum.languages;
  }
}
