import {LicensesModel} from "@app/generated-api";
import {ResourceNameSpace} from "../../../core/stores/resource/resource-namespace.model";
import {ResourceState, ResourceStateModel, defaultResourceStateInitValue} from "@app/core/stores/resource/resource.state";
import {ApiResourceNameEnum} from "@app/shared/enums/api-resource-name.enum";
import {AdminResourceApiEnum, ResourceApiEnum} from "@app/shared/enums/api.enum";
import {StateEnum} from "@app/shared/enums/state.enum";
import {LicenseAction, licenseActionNameSpace} from "@app/shared/stores/license/license.action";
import {QueryParametersModel} from "@app/shared/models/query-parameters.model";
import {ApiService} from "@app/core/http/api.service";
import {NotificationService} from "@app/core/services/notification.service";
import {Action, State, StateContext, Store} from "@ngxs/store";
import {Observable} from "rxjs";

export interface LicenseStateModel extends ResourceStateModel<LicensesModel> {
}

@State<LicenseStateModel>({
  name: StateEnum.license,
  defaults: {
    ...defaultResourceStateInitValue,
  },
})
export class LicenseState extends ResourceState<LicensesModel> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService) {
    super(apiService, store, notificationService);
  }

  defineState(): StateEnum {
    return StateEnum.license;
  }

  defineResourceNameSpace(): ResourceNameSpace {
    return licenseActionNameSpace;
  }

  defineSubResource(): ApiResourceNameEnum[] {
    return [];
  }

  defineUrlResource(): ResourceApiEnum {
    return AdminResourceApiEnum.licenses;
  }
}
