import {ApiService} from "@app/core/http/api.service";
import {NotificationService} from "@app/core/services/notification.service";
import {Relation2TiersNameSpace} from "@app/core/stores/relation-2-tiers/relation-2-tiers-namespace.model";
import {
  defaultRelation2TiersStateInitValue,
  Relation2TiersState,
  Relation2TiersStateModel,
} from "@app/core/stores/relation-2-tiers/relation-2-tiers.state";
import {OrganizationalUnitsModel, SubmissionPoliciesModel} from "@app/generated-api";
import {ApiResourceNameEnum} from "@app/shared/enums/api-resource-name.enum";
import {AdminResourceApiEnum, ResourceApiEnum} from "@app/shared/enums/api.enum";
import {StateEnum} from "@app/shared/enums/state.enum";
import {orgUnitSubmissionPolicyActionNameSpace} from "@app/shared/stores/organizational-unit/submission-policy/organizational-unit-submission-policy.action";

import {Actions, State, Store} from "@ngxs/store";

export interface OrganizationalUnitSubmissionPolicyStateModel extends Relation2TiersStateModel<SubmissionPoliciesModel> {
}

@State<OrganizationalUnitSubmissionPolicyStateModel>({
  name: StateEnum.organizationalUnitSubResourceSubmissionPolicy,
  defaults: {
    ...defaultRelation2TiersStateInitValue,
  },
})
export class OrganizationalUnitSubmissionPolicyState extends Relation2TiersState<OrganizationalUnitsModel> {

  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$);
  }

  defineUrlResource(): ResourceApiEnum {
    return AdminResourceApiEnum.organizationalUnits;
  }

  defineState(): StateEnum {
    return StateEnum.organizationalUnit;
  }

  defineRelation2TiersNameSpace(): Relation2TiersNameSpace {
    return orgUnitSubmissionPolicyActionNameSpace;
  }

  defineResourceName(): ApiResourceNameEnum {
    return ApiResourceNameEnum.SUB_POLICY;
  }
}
