import {ComponentType} from "@angular/cdk/portal";
import {OnInit} from "@angular/core";
import {MatDialog} from "@angular/material";
import {ActivatedRoute} from "@angular/router";
import {StoreUtil} from "@app/core/utils/stores/store.util";
import {ResourceNameSpace} from "../../../core/stores/resource/resource-namespace.model";
import {AbstractComponent} from "@app/shared/components/abstract/abstract.component";
import {ResourceStateModel} from "@app/core/stores/resource/resource.state";
import {AbstractDeleteDialog} from "@app/shared/dialogs/abstract-delete/abstract-delete.dialog";
import {ModelAttributeEnum} from "@app/shared/enums/model-attribute.enum";
import {StateEnum} from "@app/shared/enums/state.enum";
import {DeleteDialogModel} from "@app/shared/models/delete-dialog.model";
import {ResourceActionHelper} from "@app/core/stores/resource/resource-action.helper";
import {StoreDialogUtil} from "@app/core/utils/stores/store-dialog.util";
import {StoreRouteUtil} from "@app/core/utils/stores/store-route.util";
import {Navigate} from "@ngxs/router-plugin";
import {Store} from "@ngxs/store";
import {Observable} from "rxjs";
import {take} from "rxjs/operators";

export abstract class AbstractDetailView<T, U extends ResourceStateModel<T>> extends AbstractComponent implements OnInit {

  isLoading$: Observable<boolean>;
  current$: Observable<T>;

  abstract readonly KEY_PARAM_NAME: string;
  abstract readonly KEY_EDIT_BUTTON: string;
  abstract readonly KEY_DELETE_BUTTON: string;

  private resId: string;

  protected constructor(protected store: Store,
                        protected route: ActivatedRoute,
                        public dialog: MatDialog,
                        private state: StateEnum,
                        private resourceNameSpace: ResourceNameSpace) {
    super();
    this.isLoading$ = this.store.select(s => StoreUtil.isLoadingState(s[state] as U));
    this.current$ = this.store.select(s => (s[state] as U).current);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.store.dispatch(ResourceActionHelper.loadResource(this.resourceNameSpace));
    this.retrieveCurrentModelWithUrl();
  }

  private retrieveCurrentModelWithUrl(): void {
    this.resId = this.route.snapshot.paramMap.get("id");
    this.getById(this.resId);
  }

  private getById(id: string): void {
    this.store.dispatch(ResourceActionHelper.getById(this.resourceNameSpace, id));
    this.getSubResourceWithParentId(id);
  }

  abstract getSubResourceWithParentId(id: string): void;

  getResId(): string {
    return this.resId;
  }

  edit(): void {
    this.store.dispatch(new Navigate([StoreRouteUtil.getEditRoute(this.state), this.resId]));
  }

  delete(): void {
    const datas = {} as DeleteDialogModel;
    this.subscribe(
      this.current$.pipe(take(1)),
      (model: T) => {
        datas.name = model[this.KEY_PARAM_NAME];
        datas.resId = model[ModelAttributeEnum.resId];
      });
    const dialogRef = this.dialog.open(StoreDialogUtil.delete(this.state) as ComponentType<AbstractDeleteDialog<T, U>>, {
      width: "250px",
      data: datas,
    });
  }
}
