import {OnInit} from "@angular/core";
import {ActivatedRoute} from "@angular/router";
import {ResourceActionHelper} from "@app/core/stores/resource/resource-action.helper";
import {StoreUtil} from "@app/core/utils/stores/store.util";
import {ResourceNameSpace} from "../../../core/stores/resource/resource-namespace.model";
import {ResourceStateModel} from "@app/core/stores/resource/resource.state";
import {AbstractComponent} from "@app/shared/components/abstract/abstract.component";
import {StateEnum} from "@app/shared/enums/state.enum";
import {Store} from "@ngxs/store";
import {Observable} from "rxjs";

export abstract class AbstractEditView<T, U extends ResourceStateModel<T>> extends AbstractComponent implements OnInit {
  isLoading$: Observable<boolean>;
  current$: Observable<T>;

  private resId: string;

  protected constructor(protected store: Store,
                        protected route: ActivatedRoute,
                        private state: StateEnum,
                        private resourceNameSpace: ResourceNameSpace) {
    super();
    this.isLoading$ = this.store.select(s => StoreUtil.isLoadingState(s[state] as U));
    this.current$ = this.store.select(s => (s[state] as U).current);
  }

  ngOnInit(): void {
    this.store.dispatch(ResourceActionHelper.loadResource(this.resourceNameSpace));
    this.getCurrentModel();
  }

  private getCurrentModel(): void {
    this.resId = this.route.snapshot.paramMap.get("id");
    this.getById(this.resId);
  }

  private getById(id: string): void {
    this.store.dispatch(ResourceActionHelper.getById(this.resourceNameSpace, id));
    this.getSubResourceWithParentId(id);
  }

  abstract getSubResourceWithParentId(id: string): void;

  getResId(): string {
    return this.resId;
  }

  update(model: T): void {
    this.store.dispatch(ResourceActionHelper.update<T>(this.resourceNameSpace, model));
  }
}
