import {OnInit} from "@angular/core";
import {ResourceActionHelper} from "@app/core/stores/resource/resource-action.helper";
import {ResourceStateModel} from "@app/core/stores/resource/resource.state";
import {StoreRouteUtil} from "@app/core/utils/stores/store-route.util";
import {StoreUtil} from "@app/core/utils/stores/store.util";
import {AbstractComponent} from "@app/shared/components/abstract/abstract.component";
import {ModelAttributeEnum} from "@app/shared/enums/model-attribute.enum";
import {StateEnum} from "@app/shared/enums/state.enum";
import {DataTableColumnsModel} from "@app/shared/models/data-table-columns.model";
import {QueryParametersModel} from "@app/shared/models/query-parameters.model";
import {Navigate} from "@ngxs/router-plugin";
import {Store} from "@ngxs/store";
import {Observable} from "rxjs";
import {environment} from "../../../../environments/environment";
import {ResourceNameSpace} from "../../../core/stores/resource/resource-namespace.model";

export abstract class AbstractListView<T, U extends ResourceStateModel<T>> extends AbstractComponent implements OnInit {
  isLoading$: Observable<boolean>;
  list$: Observable<T[]>;
  queryParameters$: Observable<QueryParametersModel>;

  columns: DataTableColumnsModel<T>[];

  abstract readonly KEY_CREATE_BUTTON: string;
  abstract readonly KEY_REFRESH_BUTTON: string;

  protected constructor(protected store: Store, private state: StateEnum, private resourceNameSpace: ResourceNameSpace) {
    super();
    this.isLoading$ = this.store.select(s => StoreUtil.isLoadingState(s[state] as U));
    this.list$ = this.store.select(s => (s[state] as U).list);
    this.queryParameters$ = this.store.select(s => (s[state] as U).queryParameters);
  }

  ngOnInit(): void {
    this.defineColumns();
    this.getAll(new QueryParametersModel(environment.defaultPageSizeOption));
  }

  abstract defineColumns(): void;

  create(): void {
    this.store.dispatch(new Navigate([StoreRouteUtil.getCreateRoute(this.state)]));
  }

  getAll(queryParameters?: QueryParametersModel): void {
    this.store.dispatch(ResourceActionHelper.getAll(this.resourceNameSpace, queryParameters));
  }

  showDetail(model: T): void {
    this.store.dispatch(new Navigate([StoreRouteUtil.getDetailRoute(this.state), model[ModelAttributeEnum.resId]]));
  }

  onQueryParametersEvent(queryParameters: QueryParametersModel): void {
    this.store.dispatch(ResourceActionHelper.changeQueryParameters(this.resourceNameSpace, queryParameters));
  }
}
